import urllib.parse
import requests
import time 

CONFIG_SCHEMA_FIELD_REQUIRED = ["host", "port", "password"]
CONFIG_FIELD_DEFAULT= {"protocol": "redis", "port": 6379, "health_route": "/health"}
CONFIG_FIELD_REQUIRED = ["host", "port", "password", "protocol", "health_route"]

def validate_service_schema(config):
    for field in CONFIG_SCHEMA_FIELD_REQUIRED:
        if config.get(field) is None:
            raise Exception(f"Service: service '{config['name']}' field '{field}' is required.")

def prepare_config(config):
    for field in CONFIG_FIELD_DEFAULT:
        if config.get(field) is None:
            config[field] = CONFIG_FIELD_DEFAULT[field]

def validate_service(schema, config):
    for field in CONFIG_SCHEMA_FIELD_REQUIRED:
        if config.get(field) is None:
            raise Exception(f"Service: service '{config['name']}' field '{field}' is required.")
        if str(config[field]).strip(" ") == "" and schema[field].get("allowEmpty") != True:
            raise Exception(f"Service: service '{config['name']}' field '{field}' is invalid.")

def health_check(config):
    health_url = urllib.parse.urljoin(f"{config['protocol']}://{config['host']}:{config['port']}", config["health_route"])
    start_time = time.time()
    count = 0
    while True:
        time.sleep(config["interval"])
        if count > config["retry"]:
            raise Exception(f"Service: '{config['name']}' too many retries.")
        count += 1
        try:
            response = requests.get(health_url)
            if (response.status_code != 200):
                raise Exception("response status code")
            break
        except Exception as e:
            print(e)
            print("-"*5)
        if time.time() - start_time > config["timeout"]:
            raise Exception(f"Service: '{config['name']}' timeout exception.")


import pika
import time

CONFIG_SCHEMA_FIELD_REQUIRED = ["host", "password", "username"]
CONFIG_FIELD_DEFAULT= {"protocol": "amqp", "port": 5672, "vhost": "/"}
CONFIG_FIELD_REQUIRED = ["host", "port", "password", "username", "protocol", "vhost"]

def validate_service_schema(config):
    for field in CONFIG_SCHEMA_FIELD_REQUIRED:
        if config.get(field) is None:
            raise Exception(f"Service: service '{config['name']}' field '{field}' is required.")

def prepare_config(config):
    for field in CONFIG_FIELD_DEFAULT:
        if config.get(field) is None:
            config[field] = CONFIG_FIELD_DEFAULT[field]

def validate_service(schema, config):
    for field in CONFIG_SCHEMA_FIELD_REQUIRED:
        if config.get(field) is None:
            raise Exception(f"Service: service '{config['name']}' field '{field}' is required.")
        if str(config[field]).strip(" ") == "" and schema[field].get("allowEmpty") != True:
            raise Exception(f"Service: service '{config['name']}' field '{field}' is invalid.")

def health_check(config):
    queue_name = f'{config["name"]}-healthcheck'.lower()
    credentials = pika.PlainCredentials(config["username"], config["password"])
    conn_params = pika.ConnectionParameters(host=config["host"], port=config["port"], virtual_host = config["vhost"], credentials = credentials)
    start_time = time.time()
    count = 0
    while True:
        time.sleep(config["interval"])
        if count > config["retry"]:
            raise Exception(f"Service: '{config['name']}' too many retries.")
        count += 1
        try:
            connection = pika.BlockingConnection(conn_params)
            channel = connection.channel()
            channel.basic_qos(prefetch_count=1)
            channel.queue_declare(queue_name, durable = False, auto_delete = True)
            channel.basic_consume(queue_name, lambda x: print(x))
            connection.close()
            break
        except Exception as e:
            print(e)
            print("-"*5)
        if time.time() - start_time > config["timeout"]:
            raise Exception(f"Service: '{config['name']}' timeout exception.")


import redis 
import time

CONFIG_SCHEMA_FIELD_REQUIRED = ["host", "password"]
CONFIG_FIELD_DEFAULT= {"protocol": "redis", "port": 6379}
CONFIG_FIELD_REQUIRED = ["host", "port", "password", "protocol"]
TEST_CONNECTION_KEY = "_connection_test"

def validate_service_schema(config):
    for field in CONFIG_SCHEMA_FIELD_REQUIRED:
        if config.get(field) is None:
            raise Exception(f"Service: service '{config['name']}' field '{field}' is required.")

def prepare_config(config):
    for field in CONFIG_FIELD_DEFAULT:
        if config.get(field) is None:
            config[field] = CONFIG_FIELD_DEFAULT[field]

def validate_service(schema, config):
    for field in CONFIG_SCHEMA_FIELD_REQUIRED:
        if config.get(field) is None:
            raise Exception(f"Service: service '{config['name']}' field '{field}' is required.")
        if str(config[field]).strip(" ") == "" and schema[field].get("allowEmpty") != True:
            raise Exception(f"Service: service '{config['name']}' field '{field}' is invalid.")

def health_check(config):
    start_time = time.time()
    count = 0
    while True:
        time.sleep(config["interval"])
        if count > config["retry"]:
            raise Exception(f"Service: '{config['name']}' too many retries.")
        count += 1
        try:
            redis_connection = redis.Redis(host=config["host"], port=config["port"], password=config["password"], health_check_interval=1)
            redis_connection.ping()
            break
        except Exception as e:
            print(e)
            print("-"*5)
        if time.time() - start_time > config["timeout"]:
            raise Exception(f"Service: '{config['name']}' timeout exception.")


VERSION = "0.0.7"
FILE_TYPE = {
    "JSON": "json"
}
SERVICE_TYPE = {
    "REDIS": "redis",
    "MYSQL": "mysql",
    "INFLUXDB": "influxdb",
    "RABBIT_MQ": "rabbit_mq",
    "REST_API": "rest_api"
}
REQUEST_METHOD = {
    "GET": "GET",
    "PUT": "PUT",
    "POST": "POST",
    "DELETE": "DELETE"
}
CONFIG_FIELD_REQUIRED = [{"name": "services", "type": list}]
CONFIG_SERVICE_FIELD_REQUIRED = [{"name": "name", "type": str}, {"name": "type", "type": str}, {"name": "timeout", "type": int}, {"name": "retry", "type": int}, {"name": "interval", "type": int}]


from dotenv import load_dotenv
import copy
import json
import os
from .config import SERVICE_TYPE, CONFIG_FIELD_REQUIRED, CONFIG_SERVICE_FIELD_REQUIRED

def parse_config(config_path) -> dict:
    with open(config_path, 'r') as file:
        try:
            config = json.load(file)
        except:
            raise Exception("Config: invalid JSON.")
        return config

def resolve_config(config) -> dict:
    resolved_config = copy.deepcopy(config)
    resolved_config["services"] = []
    for service in config["services"]:
        resolved_service = {}
        for field in service.keys():
            if field in map(lambda x: x["name"],CONFIG_SERVICE_FIELD_REQUIRED):
                resolved_service[field] = service[field]
                continue
            if service[field].get("env") is not None:
                resolved_service[field] = os.getenv(service[field].get("env"))
                continue
            if service[field].get("value") is not None:
                resolved_service[field] = service[field].get("value")
                continue
        prepare_config = __import__(f'{service["type"]}', globals(), locals(), [], 1).prepare_config
        prepare_config(resolved_service)
        resolved_config["services"].append(resolved_service)
    return resolved_config

def get_config(config_path, env_path) -> dict:
    if env_path:
        load_dotenv(env_path)
    config = parse_config(config_path)
    validate_schema(config)
    resolved_config = resolve_config(config)
    validate_config(config, resolved_config)
    return resolved_config

def validate_service_schema(config) -> None:
    for field in CONFIG_SERVICE_FIELD_REQUIRED:
        if config.get(field["name"]) is None:
            raise Exception(f"Service: field '{field['name']}' is required.")
        if type(config[field["name"]]) != field["type"]:
            raise Exception(f"Service: field '{field['name']}' is invalid.")
    for field in config.keys():
        if config.get(field) is None:
            raise Exception(f"Service: field '{field}' is invalid.")
    if config["type"] not in SERVICE_TYPE.values():
        raise Exception(f"Service: type '{config['type']}' is invalid.")
    for config_type in SERVICE_TYPE.values():
        if config["type"] != config_type:
            continue
        validate = __import__(f'{config_type}', globals(), locals(), [], 1).validate_service_schema
        validate(config)
        break

def validate_schema(config) -> None:
    for field in CONFIG_FIELD_REQUIRED:
        if config.get(field["name"]) is None:
            raise Exception(f"Config: field '{field['name']}' is required.")
        if type(config[field["name"]]) != field["type"]:
            raise Exception(f"Config: field '{field['name']}' is invalid.")
    for service in config["services"]:
        validate_service_schema(service)

def validate_service(schema, config) -> None:
    for config_type in SERVICE_TYPE.values():
        if config["type"] != config_type:
            continue
        validate = __import__(f'{config_type}', globals(), locals(), [], 1).validate_service
        validate(schema, config)
        break

def validate_config(schema, config) -> None:
    for index, service in enumerate(config["services"]):
        validate_service(schema["services"][index], service)

def health_check(config):
    for service in config["services"]:
        print("\n".join(["-"*30, f"Service {service['name']}", f"timeout {service['timeout']}", f"retry {service['retry']}", "-"*15]))
        check = __import__(f'{service["type"]}', globals(), locals(), [], 1).health_check
        check(service)
        print("\n".join([f"Success {service['name']}."]))


from setuptools import setup, find_packages

def read_files(files):
    data = []
    for file in files:
        with open(file, encoding='utf-8') as f:
            data.append(f.read())
    return "\n".join(data)

long_description = read_files(['README.md'])
requirements = ['requests', 'click', 'redis', 'urllib3', 'mysql-connector-python', 'pika', 'python-dotenv']

setup(
    name = 'health-util',
    description = 'A cli package for providing healthcheck',
    long_description = long_description,
    long_description_content_type = "text/markdown",
    version = '0.0.7',
    author = 'mogu4iy',
    author_email = 'mogu4iy.kotygoroshko@gmail.com',
    license = 'MIT',
    url = "https://github.com/mogu4iy/health-util/",
    packages=find_packages(where="src"),
    package_dir={'':"src"},
    install_requires = [requirements],
    python_requires='>=3.7',
    entry_points = {
        "console_scripts": [
            "healthcheck=healthcheck.__main__:cli",
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


from genericpath import exists
import os
import sys

from .config import VERSION
from .healthcheck import get_config, parse_config, validate_schema, health_check

try:
    import click
except ImportError:
    sys.stderr.write('It seems health-util is not installed with cli option. \n'
                     'Run pip install "health-util[cli]" to fix this.')
    sys.exit(1)

@click.group(chain=True)
@click.option('-e', '--env', default=None,
              type=click.Path(file_okay=True),
              help="Path to env file, defaults to .env.healthcheck file in current working directory.")
@click.option('-c', '--config', default='healthcheck.json',
              type=click.Path(exists=True),
              help="Path to config file, defaults to healthcheck.json file in current working directory.")
@click.version_option(version=VERSION)
@click.pass_context
def cli(ctx: click.Context, env, config) -> None:
    ctx.ensure_object(dict)
    ctx.obj["ENV_PATH"] = os.path.join(os.getcwd(), env) if env else None
    ctx.obj["ENV_FILE"] = env
    ctx.obj["CONFIG_PATH"] = os.path.join(os.getcwd(), config) if config else None
    ctx.obj["CONFIG_FILE"] = config

@cli.command()
@click.pass_context
def validate(ctx: click.Context) -> None:
    config = parse_config(config_path=ctx.obj["CONFIG_PATH"])
    validate_schema(config)
    click.echo(f"Config file '{ctx.obj['CONFIG_FILE']}' is valid.")

@cli.command()
@click.pass_context
def check(ctx: click.Context) -> None:
    config = get_config(config_path=ctx.obj["CONFIG_PATH"], env_path=ctx.obj["ENV_PATH"])
    health_check(config)